import numpy as np
from UtilsRPE.Functions import load_vault


def load_database():
    from os import listdir
    dirpath = "../PrecisionNamelists"
    files = listdir(dirpath)
    database = {}
    sp = [f.split("_") for f in files]
    groups = [int(x[2][1:]) for x in sp if len(x) == 5]
    members = [int(x[3]) for x in sp if len(x) == 5]
    id = [int(x[4]) for x in sp if len(x) == 5]

    for g in groups:
        database[g] = {}

    for i in range(len(id)):
        database[groups[i]][members[i]] = id[i]
    return database


def var_id_from_group_and_member(group, member, database):
    try:
        return database[group][member]
    except KeyError:
        return None


def get_id_from_filename(_filename):
    splitted =_filename.split("/")
    try:
        group = int(splitted[0][1:])
    except ValueError:
        return None, None
    if len(splitted) == 2:
        member = None
    elif len(splitted) == 3:
        member = int(splitted[1][4:])
    else:
        raise AssertionError("MEEEEEEC")
    return group, member


def load_experiment_accuracies(var_name=None):
    if var_name is None:
        filename = "experiment_accuracies.txt"
    else:
        filename = "%s_accuracies.txt" % var_name

    with open(filename) as f:
        _data = [l for l in f]

    _names = [l.split()[0] for l in _data]
    _error = [float(l.split()[1]) for l in _data]
    # Obtain the corresponding group and member info from the name file
    _group_and_member = [get_id_from_filename(name) for name in _names]
    _group = np.array([g for g, m in _group_and_member])
    # Filter by experiments that are associated to an id
    _data = np.array([_error[i] for i in range(len(_error))])

    return _group, _data


if __name__ == "__main__":
    vault = load_vault("../../vault.pkl")

    all_variables = [var for var in vault.variables if var.id is not None]

    var_names = ["thetao", "so", "heatc", "saltc"]

    thresholds = {"thetao": 1.49*10**-6, "so": 6.41*10**-6, "heatc":2.74*10**-6, "saltc": 5.47*10**-9}

    log = ""

    # Dictionary to save the variables that are safe for every variable
    safe_dictionary = {}
    some_effect_dictionary = {}
    previous = 0
    namelist_index = 0
    for case in [6]:
        for var_name in var_names:
            # Load the variable id and the associated error from file experiment_accuracies.txt
            groups, error = load_experiment_accuracies(var_name)
            assert len(groups) > 0
            assert len(error) > 0
            # At least one of the variables in this list generates problems.

            threshold = thresholds[var_name]/case
            safe_ids = groups[error < threshold]
            safe_dictionary[var_name] = safe_ids
            some_effect = groups[error > 0.0]
            print(var_name, len(some_effect))
            some_effect_dictionary[var_name] = some_effect

            # print("Var: %s Threshold: %.2e %i/%i" % (var_name, threshold, len(safe_ids), len(ids)))

